// implicit_euler.cc
#include <fstream>
#include <cmath>
#include <vector>
typedef typename std::vector<double> doubleVec;
double f(double t, double y){
  return cos(2.0*t)*pow(y,2.0);
}
double Phi(double t, double y_old, double y_new, double tau){
  return f(t+tau,y_new);
} 
void save_solution(doubleVec y, int K){
  std::fstream f;
  f.open("sol.dat",std::ios::out);
  if (f.good()){
    for (int k=0; k<=K; k++){
      f << y[k] << "\n";
    }
  }
  f.close();
}
int main(){
  double y_0 = 1.0, T = 10.0, tau = 1.0/100.0, t_k;
  double z, z_new, diff, eps_stop = tau/10;
  int k, K = floor(T/tau);
  doubleVec y(K+1);
  y[0] = y_0;
  for (k=0; k<K; k++){
    t_k = k*tau;
    z = y[k];
    diff = 1.0;
    while (diff>eps_stop){
      z_new = y[k]+tau*Phi(t_k,y[k],z,tau);
      diff = fabs(z_new-z);
      z = z_new;
    }
    y[k+1] = z;
  }
  save_solution(y,K);
}



