function implicit_euler
y_0 = 1; T = 10;
tau = 1/100; K = floor(T/tau);
y(inc(0)) = y_0;
for k = 0:K-1
    t_k = k*tau;
    y(inc(k+1)) = fixed_point_iteration(t_k,y(inc(k)),tau);
    % y(inc(k+1)) = newton_iteration(t_k,y(inc(k)),tau);
end
plot(tau*(0:K),y);

function z = fixed_point_iteration(t,y_old,tau)
z = y_old; diff = 1; eps_stop = tau/10; 
while diff > eps_stop
    z_new = y_old+tau*Phi(t,y_old,z,tau);
    diff = abs(z_new-z);
    z = z_new;
end

function z = newton_iteration(t,y_old,tau)
z = y_old; diff = 1; eps_stop = tau/10;
while diff > eps_stop
    F = z-y_old-tau*Phi(t,y_old,z,tau);
    dF = 1-tau*dPhi_y(t,y_old,z,tau);
    z_new = z-F/dF;
    diff = abs(z_new-z);
    z = z_new;
end

function val = Phi(t,y_old,y_new,tau)
val = f(t+tau,y_new);

function val = dPhi_y(t,y_old,y_new,tau)
val = df_y(t+tau,y_new);

function val = f(t,y)
val = cos(2*t)*y^2;

function val = df_y(t,y)
val = cos(2*t)*2*y;

function val = inc(k)
val = k+1;
